#!/usr/bin/python

import sys
import re
import csv

class FPGAPin:
    def __init__(self, name, bank, pad):
        self.name = name
        self.bank = int(bank)
        self.pad = pad
        self.in_use = False
        self.assigned_signal = None

def differ_by_single_letter(s1, s2):
    if len(s1) != len(s2):
        return False;

    ch = 0
    last = None
    for i in range(0, len(s1)):
        if s1[i] != s2[i]:
            ch += 1
            last = i
    if ch == 1:
        return ( s1[last], s2[last] )
    else:
        return False

class FPGAPinFile:
    def __init__(self, filename):
        self.pins = []
        for l in open(filename).readlines():
            tok = l.split()
            if len(tok) < 8:
                continue

            if tok[6] == "HR" or tok[6] == "HP" or tok[6] == "GTX" or tok[6]=="GTP":
                self.pins += [ FPGAPin(tok[1], tok[3], tok[0])]

    def find_pin(self, name, banks=None):
        rv = None
        for p in self.pins:
            if banks and not p.bank in banks:
                continue
            if p.name == name:
                return p
        return None

    def first_free_pin(self, banks=None):
        rv = None
        for p in self.pins:
            if p.in_use:
                continue
            if banks and not p.bank in banks:
                continue
            return p

    def first_free_diff_pair(self, banks=None):
        rv = None
        
        for p_ref in self.pins:
            if p_ref.in_use:
                continue
            id1 = re.match("IO_L(\d+)([PN]).+", p_ref.name)
            if id1 == None:
                continue
            
            p_ref_n = id1.group(1)
            p_ref_pol = id1.group(2)
            
            for p in self.pins:
                if p.in_use:
                    continue
                if banks and not p.bank in banks:
                    continue
                if p.bank != p_ref.bank:
                    continue
                id2 = re.match("IO_L(\d+)([PN]).+", p.name)
                if id2 == None:
                    continue


                p_n = id2.group(1)
                p_pol = id2.group(2)
                
                if p_ref_n == p_n and p_ref_pol != p_pol:
                #    print("MAtch!")
                    if p_ref_pol.lower() == 'p':
                        return ( p_ref, p )
                    else:
                        return ( p, p_ref )

        return rv

    def free_pins_in_banks(self, banks):
        for b in banks.banks:
            cnt_free = 0
            cnt_total = 0
            for p in self.pins:
                if p.bank == b:
                    cnt_total += 1
                    if not p.in_use:
                        cnt_free += 1
            print("Bank %d: %d total, %d free pins" % (b, cnt_total, cnt_free))

class Signal: 
    def __init__(self, name, range_hi = 0, range_lo = 0, diff=False, force=None, banks=None, iostandard=None):
        self.name = name
        self.range_hi = range_hi
        self.range_lo = range_lo
        self.diff = diff
        self.force = force
        self.banks = None
        if banks:
            self.banks = banks.banks
            banks.pins.append( self )
        self.iostandard = iostandard
        self.done=False
        self.pair=None
        self.fpga_pins={}
        

class TopLevelSignals:
    def __init__(self):
        self.signals=[]

    def add( self, name, range_hi = 0, range_lo = 0, diff=False, force=None, banks=None, iostandard=None):
        self.signals += [ Signal( name, range_hi, range_lo, diff, force, banks, iostandard)]

    def reserve_pin( self, name ):
        self.signals += [ Signal( "Reserved", 0, 0, False, force=name ) ]


    def print_stats(self):
        total = 0
        for s in self.signals:
            total += (s.range_hi + 1 - s.range_lo)

        print("Total signals : %d", total)

    def find_diff_pair(self, p_ref):
        rv = None
        
        for p in self.signals:
            ch = 0
            #print("%s %s", p.name, p_ref.name)
            if len(p.name) != len(p_ref.name):
                continue
            for i in range(0, len(p_ref.name)):
                #print ( p.name[i].lower() )
                if p.name[i].lower() != p_ref.name[i].lower():
                    ch+=1
                if p.name[i].lower()=="p" and p_ref.name[i].lower()=="n":
                    #print("MATCH")
                    rv = (p, p_ref)
                elif p.name[i].lower()=="n" and p_ref.name[i].lower()=="p":
                    #print("MATCH2")
                    rv = (p_ref, p)
            if ch == 1 and rv:
                return rv

        return rv


class Design:
    def __init__(self, pin_table_file):
        self.fpga_pins = FPGAPinFile( pin_table_file )
        self.signals = TopLevelSignals()

    def allocate_pins(self):

        for s in self.signals.signals:
            if s.done:
                continue
            if s.force:
                p = self.fpga_pins.find_pin(s.force)
                if p == None:
                    print("!!!! unknown forced pin")
                print("Force: %-30s -> %-20s" % (s.name, s.force))
                s.done = True
                s.fpga_pins[0] = p
                p.in_use = True
                p.assigned_signal = s

        print("Free pins post forced placement: ")
        pins_in_banks={}
        n_forces = 0
        n_free = 0
        for p in self.fpga_pins.pins:
            if p.assigned_signal == None:
                if not p.bank in pins_in_banks.keys():
                    pins_in_banks[ p.bank ] = []

                pins_in_banks[p.bank].append(p)

        for b in pins_in_banks.keys():
            print("- bank %d: %d pins" % (b, len( pins_in_banks[b]) ) )


        for s in self.signals.signals:
            if s.done:
                continue
            if s.diff:
                for idx in range( s.range_lo, s.range_hi + 1 ):
                    #print("Do Signal %s[%d]" % (s.name, idx))
                    pair = self.signals.find_diff_pair( s )
                    f_pair = self.fpga_pins.first_free_diff_pair( banks = s.banks )
                    if f_pair == None:
                        print("!!!! Unable to fit diff pair: %s" % s.name)
                        break
                    pair[0].done = True
                    pair[1].done = True
                    f_pair[0].in_use = True
                    f_pair[1].in_use = True

                    f_pair[0].assigned_signal = pair[0]
                    f_pair[1].assigned_signal = pair[1]
            
                    pair[0].fpga_pins[idx] = f_pair[0]
                    pair[1].fpga_pins[idx] = f_pair[1]

                    #print("DP %s[%d] %s %s %d" %( pair[0].name, idx, f_pair[0].name, f_pair[0].pad, f_pair[0].bank ) )
                    #print("DN %s[%d] %s %s %d" %( pair[1].name, idx, f_pair[1].name, f_pair[1].pad, f_pair[1].bank ) )
            else:
                for idx in range( s.range_lo, s.range_hi + 1 ):
                    f_pin = self.fpga_pins.first_free_pin( banks = s.banks )
                    if f_pin == None:
                        total = 0
                        print("!!!! Unable to fit pin: %s [%d]" % (s.name, idx))
                        for p in self.fpga_pins.pins:
                            if p.bank in s.banks:
                                print("Bank %d pin %s -> %s %d" % (p.bank, p.name, p.assigned_signal.name, p.in_use))
                                total+=1
                        print("Total used pins: %d" % total)
                        return
                        break
                    s.done = True
                    f_pin.in_use = True
                    f_pin.assigned_signal = s
                    s.fpga_pins[idx] = f_pin

            #print("D %s[%d] %s %s %d" %( s.name, idx, f_pin.name, f_pin.pad, f_pin.bank ) )

    def write_xdc(self, filename="out.xdc"):
        f_xdc = open(filename,"wb")
        for s in self.signals.signals:
            if s.name == "Reserved":
                continue
            print ( "process %s "  % s.name )
            for idx in range( s.range_lo, s.range_hi + 1 ):
                if s.range_lo == 0 and s.range_hi == 0:
                    rstring = ""
                else:
                    rstring = "[%d]" % idx
                f_xdc.write("# PadFunction: %s\n"%s.fpga_pins[idx].name)
                f_xdc.write("set_property PACKAGE_PIN %s [get_ports {%s%s}]\n" % (s.fpga_pins[idx].pad, s.name, rstring))
                iostd = None
                if s.iostandard != None:
                    iostd = s.iostandard
                else:
                    iostd = "LVCMOS33"
                if iostd:
                    f_xdc.write("set_property IOSTANDARD %s [get_ports {%s%s}]\n"% (iostd, s.name, rstring))

    def show_pin_count_in_banks(self, bgroup):
        for s in self.signals.signals:
            if s.banks == bgroup:
                print("- %s" % s.name)

class IOBankGroup:
    def __init__(self,banks):
        self.banks=banks
        self.pins=[]

    def show_pins(self):
        total = 0
        print("Bank Group: %s" % str( self.banks ))
        for p in self.pins:
            cnt = p.range_hi - p.range_lo + 1
            print("- %-40s [%-3d:%-3d] -> %-3d pin(s)" % (p.name, p.range_lo, p.range_hi, p.range_hi - p.range_lo + 1))
            total += cnt
        print("Total pins: %d" % total)
    

design = Design('xc7k160tfbg676pkg.txt')

# all banks: 12..16, 32..36
all_banks = IOBankGroup( [12,13,14,15,16,32,33,34] )
banks_ddr3 = IOBankGroup( [32, 33, 34] )
banks_fmc = IOBankGroup([ 12, 13, 15 ])
banks_33 = IOBankGroup([ 14, 16 ])

#IO_L1N_T0_D01_DIN_14
#IO_L1P_T0_D00_MOSI_14
#IO_L6P_T0_FCS_B_14

has_serial_vme_bridge = True

design.signals.add("clk_20m_vcxo_i", force="IO_L12P_T1_MRCC_14",iostandard="LVCMOS25")
design.signals.add("clk_125m_pllref_p_i", force="IO_L13P_T2_MRCC_14", iostandard="LVDS_25")
design.signals.add("clk_125m_pllref_n_i", force="IO_L13N_T2_MRCC_14", iostandard="LVDS_25")
design.signals.add("clk_125m_gtx_n_i", force="MGTREFCLK0N_115")
design.signals.add("clk_125m_gtx_p_i", force="MGTREFCLK0P_115")
design.signals.add("clk_fpga2_p_i", force="IO_L13P_T2_MRCC_16", iostandard="LVDS_25")
design.signals.add("clk_fpga2_n_i", force="IO_L13N_T2_MRCC_16", iostandard="LVDS_25")
design.signals.add("clk_si57x_p_i", force="IO_L12P_T1_MRCC_16", iostandard="LVDS_25")
design.signals.add("clk_si57x_n_i", force="IO_L12N_T1_MRCC_16", iostandard="LVDS_25")

# reserve the VRef pins
#design.signals.reserve_pin("IO_L6N_T0_VREF_15")
#design.signals.reserve_pin("IO_L19N_T3_A21_VREF_15")
design.signals.reserve_pin("IO_L6N_T0_VREF_16")
design.signals.reserve_pin("IO_L19N_T3_VREF_16")
#design.signals.reserve_pin("IO_L6N_T0_VREF_12")
#design.signals.reserve_pin("IO_L19N_T3_VREF_12")
#design.signals.reserve_pin("IO_L6N_T0_VREF_13")
#design.signals.reserve_pin("IO_L19N_T3_VREF_13")
design.signals.reserve_pin("IO_L6N_T0_D08_VREF_14")
design.signals.reserve_pin("IO_L19N_T3_A09_D25_VREF_14")

design.signals.reserve_pin("IO_L1N_T0_34")
design.signals.reserve_pin("IO_L2P_T0_34")
design.signals.reserve_pin("IO_L2N_T0_34")
design.signals.reserve_pin("IO_L4P_T0_34")
design.signals.reserve_pin("IO_L4N_T0_34")
design.signals.reserve_pin("IO_L5P_T0_34")
design.signals.reserve_pin("IO_L5N_T0_34")
design.signals.reserve_pin("IO_L6P_T0_34")
design.signals.reserve_pin("IO_L7N_T1_34")
design.signals.reserve_pin("IO_L8P_T1_34")
design.signals.reserve_pin("IO_L8N_T1_34")
design.signals.reserve_pin("IO_L10P_T1_34")
design.signals.reserve_pin("IO_L10N_T1_34")
design.signals.reserve_pin("IO_L11P_T1_SRCC_34")
design.signals.reserve_pin("IO_L11N_T1_SRCC_34")
design.signals.reserve_pin("IO_L12P_T1_MRCC_34")
design.signals.reserve_pin("IO_L13P_T2_MRCC_34")
design.signals.reserve_pin("IO_L13N_T2_MRCC_34")
design.signals.reserve_pin("IO_L14P_T2_SRCC_34")
design.signals.reserve_pin("IO_L14N_T2_SRCC_34")
design.signals.reserve_pin("IO_L16N_T2_34")
design.signals.reserve_pin("IO_L17P_T2_34")
design.signals.reserve_pin("IO_L17N_T2_34")
design.signals.reserve_pin("IO_L18P_T2_34")
design.signals.reserve_pin("IO_L20P_T3_34")
design.signals.reserve_pin("IO_L20N_T3_34")
design.signals.reserve_pin("IO_L22P_T3_34")
design.signals.reserve_pin("IO_L22N_T3_34")
design.signals.reserve_pin("IO_L23P_T3_34")
design.signals.reserve_pin("IO_L23N_T3_34")
design.signals.reserve_pin("IO_L24P_T3_34")
design.signals.reserve_pin("IO_L24N_T3_34")
design.signals.reserve_pin("IO_L1N_T0_32")
design.signals.reserve_pin("IO_L2P_T0_32")
design.signals.reserve_pin("IO_L2N_T0_32")
design.signals.reserve_pin("IO_L4P_T0_32")
design.signals.reserve_pin("IO_L4N_T0_32")
design.signals.reserve_pin("IO_L5P_T0_32")
design.signals.reserve_pin("IO_L5N_T0_32")
design.signals.reserve_pin("IO_L6P_T0_32")
design.signals.reserve_pin("IO_L7N_T1_32")
design.signals.reserve_pin("IO_L8P_T1_32")
design.signals.reserve_pin("IO_L8N_T1_32")
design.signals.reserve_pin("IO_L10P_T1_32")
design.signals.reserve_pin("IO_L10N_T1_32")
design.signals.reserve_pin("IO_L11P_T1_SRCC_32")
design.signals.reserve_pin("IO_L11N_T1_SRCC_32")
design.signals.reserve_pin("IO_L12P_T1_MRCC_32")
design.signals.reserve_pin("IO_L13P_T2_MRCC_32")
design.signals.reserve_pin("IO_L13N_T2_MRCC_32")
design.signals.reserve_pin("IO_L14P_T2_SRCC_32")
design.signals.reserve_pin("IO_L14N_T2_SRCC_32")
design.signals.reserve_pin("IO_L16N_T2_32")
design.signals.reserve_pin("IO_L17P_T2_32")
design.signals.reserve_pin("IO_L17N_T2_32")
design.signals.reserve_pin("IO_L18P_T2_32")
design.signals.reserve_pin("IO_L20P_T3_32")
design.signals.reserve_pin("IO_L20N_T3_32")
design.signals.reserve_pin("IO_L22P_T3_32")
design.signals.reserve_pin("IO_L22N_T3_32")
design.signals.reserve_pin("IO_L23P_T3_32")
design.signals.reserve_pin("IO_L23N_T3_32")
design.signals.reserve_pin("IO_L24P_T3_32")
design.signals.reserve_pin("IO_L24N_T3_32")
design.signals.reserve_pin("IO_L1P_T0_33")
design.signals.reserve_pin("IO_L1N_T0_33")
design.signals.reserve_pin("IO_L2P_T0_33")
design.signals.reserve_pin("IO_L2N_T0_33")
design.signals.reserve_pin("IO_L4P_T0_33")
design.signals.reserve_pin("IO_L4N_T0_33")
design.signals.reserve_pin("IO_L5P_T0_33")
design.signals.reserve_pin("IO_L5N_T0_33")
design.signals.reserve_pin("IO_L6P_T0_33")
design.signals.reserve_pin("IO_L6N_T0_VREF_33")
design.signals.reserve_pin("IO_L7P_T1_33")
design.signals.reserve_pin("IO_L7N_T1_33")
design.signals.reserve_pin("IO_L8P_T1_33")
design.signals.reserve_pin("IO_L8N_T1_33")
design.signals.reserve_pin("IO_L9P_T1_DQS_33")
design.signals.reserve_pin("IO_L9N_T1_DQS_33")
design.signals.reserve_pin("IO_L10P_T1_33")
design.signals.reserve_pin("IO_L10N_T1_33")
design.signals.reserve_pin("IO_L11P_T1_SRCC_33")
design.signals.reserve_pin("IO_L11N_T1_SRCC_33")
design.signals.reserve_pin("IO_L12N_T1_MRCC_34")
design.signals.reserve_pin("IO_L15P_T2_DQS_33")
design.signals.reserve_pin("IO_L15N_T2_DQS_33")
design.signals.reserve_pin("IO_L12P_T1_MRCC_33")
design.signals.reserve_pin("IO_L1P_T0_34")
design.signals.reserve_pin("IO_L7P_T1_34")
design.signals.reserve_pin("IO_L16P_T2_34")
design.signals.reserve_pin("IO_L19P_T3_34")
design.signals.reserve_pin("IO_L1P_T0_32")
design.signals.reserve_pin("IO_L7P_T1_32")
design.signals.reserve_pin("IO_L16P_T2_32")
design.signals.reserve_pin("IO_L19P_T3_32")
design.signals.reserve_pin("IO_L3P_T0_DQS_34")
design.signals.reserve_pin("IO_L3N_T0_DQS_34")
design.signals.reserve_pin("IO_L9P_T1_DQS_34")
design.signals.reserve_pin("IO_L9N_T1_DQS_34")
design.signals.reserve_pin("IO_L15P_T2_DQS_34")
design.signals.reserve_pin("IO_L15N_T2_DQS_34")
design.signals.reserve_pin("IO_L21P_T3_DQS_34")
design.signals.reserve_pin("IO_L21N_T3_DQS_34")
design.signals.reserve_pin("IO_L3P_T0_DQS_32")
design.signals.reserve_pin("IO_L3N_T0_DQS_32")
design.signals.reserve_pin("IO_L9P_T1_DQS_32")
design.signals.reserve_pin("IO_L9N_T1_DQS_32")
design.signals.reserve_pin("IO_L15P_T2_DQS_32")
design.signals.reserve_pin("IO_L15N_T2_DQS_32")
design.signals.reserve_pin("IO_L21P_T3_DQS_32")
design.signals.reserve_pin("IO_L21N_T3_DQS_32")
design.signals.reserve_pin("IO_L3P_T0_DQS_33")
design.signals.reserve_pin("IO_L3N_T0_DQS_33")

design.signals.reserve_pin("IO_0_VRN_32")
design.signals.reserve_pin("IO_25_VRP_32")
design.signals.reserve_pin("IO_25_VRP_33")
design.signals.reserve_pin("IO_0_VRN_33")
design.signals.reserve_pin("IO_25_VRP_34")
design.signals.reserve_pin("IO_0_VRN_34")

design.signals.reserve_pin("IO_L14P_T2_SRCC_33")
design.signals.reserve_pin("IO_L13N_T2_MRCC_33")

design.signals.reserve_pin("IO_L6N_T0_VREF_32")
design.signals.reserve_pin("IO_L19N_T3_VREF_32")
design.signals.reserve_pin("IO_L6N_T0_VREF_33")
design.signals.reserve_pin("IO_L19N_T3_VREF_33")
design.signals.reserve_pin("IO_L6N_T0_VREF_34")
design.signals.reserve_pin("IO_L19N_T3_VREF_34")
design.signals.reserve_pin("IO_L13P_T2_MRCC_33")

design.signals.add("fmc0_clk_m2c_p_i[0]", force="IO_L12P_T1_MRCC_12", banks=banks_fmc, iostandard="LVDS_25")
design.signals.add("fmc0_clk_m2c_n_i[0]", force="IO_L12N_T1_MRCC_12", banks=banks_fmc, iostandard="LVDS_25")
design.signals.add("fmc0_clk_m2c_p_i[1]", force="IO_L13P_T2_MRCC_12", banks=banks_fmc, iostandard="LVDS_25")
design.signals.add("fmc0_clk_m2c_n_i[1]", force="IO_L13N_T2_MRCC_12", banks=banks_fmc, iostandard="LVDS_25")

design.signals.add("fmc1_clk_m2c_p_i[0]", force="IO_L12P_T1_MRCC_13", banks=banks_fmc, iostandard="LVDS_25")
design.signals.add("fmc1_clk_m2c_n_i[0]", force="IO_L12N_T1_MRCC_13", banks=banks_fmc, iostandard="LVDS_25")
design.signals.add("fmc1_clk_m2c_p_i[1]", force="IO_L13P_T2_MRCC_13", banks=banks_fmc, iostandard="LVDS_25")
design.signals.add("fmc1_clk_m2c_n_i[1]", force="IO_L13P_T2_MRCC_13", banks=banks_fmc, iostandard="LVDS_25")

design.signals.add("fmc0_la_p_b", 33, 0, diff=True, banks=banks_fmc, iostandard="LVDS_25")
design.signals.add("fmc0_la_n_b", 33, 0, diff=True, banks=banks_fmc, iostandard="LVDS_25")
design.signals.add("fmc1_la_p_b", 33, 0, diff=True, banks=banks_fmc, iostandard="LVDS_25")
design.signals.add("fmc1_la_n_b", 33, 0, diff=True, banks=banks_fmc, iostandard="LVDS_25")

design.signals.add("p2_b", 39, 0, diff=False, banks=banks_33, iostandard="LVCMOS25")

design.signals.add("rst_n_i", banks = banks_33, iostandard="LVCMOS25")

if has_serial_vme_bridge:
    design.signals.add("vme_sysreset_n_i", banks=banks_33, iostandard="LVCMOS25")
    design.signals.add("sfpga_rx_i", 4, 0, banks=banks_ddr3, iostandard="SSTL2")
    design.signals.add("sfpga_tx_o", 4, 0, banks=banks_ddr3, iostandard="SSTL2")
    design.signals.add("sfpga_clk_p_o", 0, 0, diff=True, banks=banks_ddr3, iostandard="DIFF_SSTL2")
    design.signals.add("sfpga_clk_n_o", 0, 0, diff=True, banks=banks_ddr3, iostandard="DIFF_SSTL2")
else:
    design.signals.add("vme_write_n_i", banks = banks_33)
    design.signals.add("vme_sysreset_n_i", banks = banks_33)
    design.signals.add("vme_retry_oe_o", banks = banks_33)
    design.signals.add("vme_retry_n_o", banks = banks_33)
    design.signals.add("vme_lword_n_b", banks = banks_33)
    design.signals.add("vme_iackout_n_o", banks = banks_33)
    design.signals.add("vme_iackin_n_i", banks = banks_33)
    design.signals.add("vme_iack_n_i", banks = banks_33)
    design.signals.add("vme_gap_i", banks = banks_33)
    design.signals.add("vme_dtack_oe_o", banks = banks_33)
    design.signals.add("vme_dtack_n_o", banks = banks_33)
    design.signals.add("vme_ds_n_i", 1, 0, banks = banks_33)
    design.signals.add("vme_data_oe_n_o", banks = banks_33)
    design.signals.add("vme_data_dir_o", banks = banks_33)
    design.signals.add("vme_berr_o", banks = banks_33)
    design.signals.add("vme_as_n_i", banks = banks_33)
    design.signals.add("vme_addr_oe_n_o", banks = banks_33)
    design.signals.add("vme_addr_dir_o", banks = banks_33)
    design.signals.add("vme_irq_o",        7, 1, banks = banks_33)
    design.signals.add("vme_ga_i",         4, 0, banks = banks_33)
    design.signals.add("vme_data_b",       31, 0, banks = banks_33)
    design.signals.add("vme_am_i",         5, 0, banks = banks_33)
    design.signals.add("vme_addr_b",       31, 1, banks = banks_33)
    design.signals.add("vme_noga_i",       4, 0, banks = banks_33)
    design.signals.add("vme_use_ga_i", banks = banks_fmc),

design.signals.add("pll20dac_din_o", banks=banks_33, iostandard="LVCMOS25"),
design.signals.add("pll20dac_sclk_o", banks=banks_33, iostandard="LVCMOS25"),
design.signals.add("pll20dac_sync_n_o", banks=banks_33, iostandard="LVCMOS25"),
design.signals.add("pll25dac_din_o", banks=banks_33, iostandard="LVCMOS25"),
design.signals.add("pll25dac_sclk_o", banks=banks_33, iostandard="LVCMOS25"),
design.signals.add("pll25dac_sync_n_o", banks=banks_33, iostandard="LVCMOS25"),

design.signals.add("sfp_txp_o", force="MGTXTXP0_115")
design.signals.add("sfp_txn_o", force="MGTXTXN0_115")
design.signals.add("sfp_rxp_i", force="MGTXRXP0_115")
design.signals.add("sfp_rxn_i", force="MGTXRXN0_115")

design.signals.add("fmc0_dp_txp_o[3]", force="MGTXTXP3_116")
design.signals.add("fmc0_dp_txp_o[2]", force="MGTXTXP2_116")
design.signals.add("fmc0_dp_txp_o[1]", force="MGTXTXP1_116")
design.signals.add("fmc0_dp_txp_o[0]", force="MGTXTXP0_116")

design.signals.add("fmc0_dp_rxp_i[3]", force="MGTXRXP3_116")
design.signals.add("fmc0_dp_rxp_i[2]", force="MGTXRXP2_116")
design.signals.add("fmc0_dp_rxp_i[1]", force="MGTXRXP1_116")
design.signals.add("fmc0_dp_rxp_i[0]", force="MGTXRXP0_116")

design.signals.add("fmc0_dp_clk_p_i", force="MGTREFCLK0P_116")

design.signals.add("fmc1_dp_txp_o", force="MGTXTXP1_115")
design.signals.add("fmc1_dp_rxp_i", force="MGTXRXP1_115")

# front panel DP ports
design.signals.add("dp0_txp_o", force="MGTXTXP2_115")
design.signals.add("dp0_rxp_i", force="MGTXRXP2_115")
design.signals.add("dp1_txp_o", force="MGTXTXP3_115")
design.signals.add("dp1_rxp_i", force="MGTXRXP3_115")

design.signals.add("clk_125m_gtx_p_i", force="MGTREFCLK0P_115")
design.signals.add("fmc1_dp_clk_p_i", force="MGTREFCLK1P_115")

design.signals.add("flash_miso_i", force="IO_L1N_T0_D01_DIN_14", iostandard="LVCMOS25")
design.signals.add("flash_mosi_o", force="IO_L1P_T0_D00_MOSI_14", iostandard="LVCMOS25")
design.signals.add("flash_cs_n_o", force="IO_L6P_T0_FCS_B_14", iostandard="LVCMOS25")

#slow signals for the serial reg
#design.signals.add("sfp_mod_def0_i", banks=banks_33),
#design.signals.add("sfp_rate_select_o", banks=banks_33),
#design.signals.add("sfp_tx_fault_i", banks=banks_ddr3),
#design.signals.add("sfp_tx_disable_o", banks=banks_33),
#design.signals.add("sfp_los_i", banks=banks_ddr3),
#design.signals.add("pcbrev_i", 4, 0, banks=banks_ddr3),
#design.signals.add("fp_led_line_oen_o", 1,0, banks=banks_ddr3)
#design.signals.add("fp_led_line_o", 1,0, banks=banks_ddr3)
#design.signals.add("fp_led_column_o",   3,0, banks=banks_ddr3)
#design.signals.add("fp_term_en_o",4, 1, banks=banks_ddr3)
#design.signals.add("fp_gpio1_a2b_o", banks=banks_ddr3)
#design.signals.add("fp_gpio2_a2b_o", banks=banks_ddr3)
#design.signals.add("fp_gpio34_a2b_o", banks=banks_ddr3)
#design.signals.add("fp_pushbutton_n_i", banks=banks_ddr3),
#design.signals.add("fmc0_pg_i", banks=banks_ddr3)
#design.signals.add("fmc0_prsnt_m2c_n_i", banks=banks_ddr3)
design.signals.add("fmc0_tck_o", banks=banks_33, iostandard="LVCMOS25")
design.signals.add("fmc0_tms_o", banks=banks_33, iostandard="LVCMOS25")
design.signals.add("fmc0_tdi_o", banks=banks_33, iostandard="LVCMOS25")
design.signals.add("fmc0_tdo_i", banks=banks_33, iostandard="LVCMOS25")
#design.signals.add("fmc1_pg_i", banks=banks_ddr3)
#design.signals.add("fmc1_prsnt_m2c_n_i", banks=banks_ddr3)
design.signals.add("fmc1_tck_o", banks=banks_33, iostandard="LVCMOS25")
design.signals.add("fmc1_tms_o", banks=banks_33, iostandard="LVCMOS25")
design.signals.add("fmc1_tdi_o", banks=banks_33, iostandard="LVCMOS25")
design.signals.add("fmc1_tdo_i", banks=banks_33, iostandard="LVCMOS25")
#design.signals.add("si57x_oe_o", banks=banks_33)

design.signals.add("ioexp_reset_o", banks=banks_33, iostandard="LVCMOS25")
design.signals.add("ioexp_sclk_o", banks=banks_33, iostandard="LVCMOS25")
design.signals.add("ioexp_rclk_o", banks=banks_33, iostandard="LVCMOS25")
design.signals.add("ioexp_d_o", banks=banks_33, iostandard="LVCMOS25")
design.signals.add("ioexp_d_i", banks=banks_33, iostandard="LVCMOS25")

design.signals.add("sfp_mod_def1_b", banks=banks_33, iostandard="LVCMOS25"),
design.signals.add("sfp_mod_def2_b", banks=banks_33, iostandard="LVCMOS25"),

design.signals.add("carrier_scl_b", banks=banks_33, iostandard="LVCMOS25"),
design.signals.add("carrier_sda_b", banks=banks_33, iostandard="LVCMOS25"),
design.signals.add("onewire_b", banks=banks_33, iostandard="LVCMOS25"),
design.signals.add("uart_rxd_i", banks=banks_33, iostandard="LVCMOS25"),
design.signals.add("uart_txd_o", banks=banks_33, iostandard="LVCMOS25"),
#design.signals.add("spi_ncs_o", banks=banks_33),
#design.signals.add("spi_mosi_o", banks=banks_33),
#design.signals.add("spi_miso_i", banks=banks_33),
design.signals.add("ddr3_spd_scl_b", banks=banks_33, iostandard="LVCMOS25"),
design.signals.add("ddr3_spd_sda_b", banks=banks_33, iostandard="LVCMOS25"),


design.signals.add("fp_gpio1_b", banks=banks_33, iostandard="LVCMOS25"),
design.signals.add("fp_gpio2_b", banks=banks_33, iostandard="LVCMOS25"),
design.signals.add("fp_gpio3_b", banks=banks_33, iostandard="LVCMOS25", force="IO_L14P_T2_SRCC_14"),
design.signals.add("fp_gpio4_b", banks=banks_33, iostandard="LVCMOS25"),

design.signals.add("fmc0_scl_b", banks=banks_33, iostandard="LVCMOS25")
design.signals.add("fmc0_sda_b", banks=banks_33, iostandard="LVCMOS25")

design.signals.add("fmc1_scl_b", banks=banks_33, iostandard="LVCMOS25")
design.signals.add("fmc1_sda_b", banks=banks_33, iostandard="LVCMOS25")

#design.signals.add("ddr_a_o",   14, 0, banks=banks_ddr3)
#design.signals.add("ddr_ba_o",  2, 0, banks=banks_ddr3)
#design.signals.add("ddr_cas_n_o", banks=banks_ddr3)
#design.signals.add("ddr_ck_n_o",diff=True, banks=banks_ddr3)
#design.signals.add("ddr_ck_p_o",diff=True, banks=banks_ddr3)
#design.signals.add("ddr_cke_o", banks=banks_ddr3)
#design.signals.add("ddr_dq_b",  63, 0, banks=banks_ddr3)
#design.signals.add("ddr_dqs_n_b",   7, 0, diff=True, banks=banks_ddr3)
#design.signals.add("ddr_dqs_p_b",   7, 0, diff=True, banks=banks_ddr3)
#design.signals.add("ddr_odt_o", banks=banks_ddr3)
#design.signals.add("ddr_ras_n_o", banks=banks_ddr3)
#design.signals.add("ddr_reset_n_o", banks=banks_ddr3)
#design.signals.add("ddr_rzq_b", banks=banks_ddr3)
#design.signals.add("ddr_dm_o",  7, 0, banks=banks_ddr3)
#design.signals.add("ddr_we_n_o", banks=banks_ddr3)
#design.signals.add("ddr_cs_n_o", banks=banks_ddr3)
design.signals.add("si57x_scl_b", banks=banks_33, iostandard="LVCMOS25")
design.signals.add("si57x_sda_b", banks=banks_33, iostandard="LVCMOS25")
design.signals.add("si57x_tune_o", banks=banks_33, iostandard="LVCMOS25")

banks_33.show_pins()
banks_ddr3.show_pins()
banks_fmc.show_pins()
design.allocate_pins()

design.fpga_pins.free_pins_in_banks(all_banks)
design.write_xdc("svec7.xdc")
#design.signals.print_stats()

