library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.NUMERIC_STD.all;

use work.gencores_pkg.all;
use work.vme64x_pkg.all;

entity svec_sfpga_bridge is
  generic(
    g_clock_period : integer
    );
  port (

    afpga_rst_n_i : in STD_LOGIC;
    afpga_clk_i   : in std_logic;

    afpga_d_i     : in std_logic_vector(7 downto 0);
    afpga_frame_i : in std_logic;
    afpga_d_o     : in std_logic_vector(7 downto 0);
    afpga_frame_o : in std_logic;

    vme_i           : in  t_vme64x_in;
    vme_o           : out t_vme64x_out
    );

end svec_sfpga_bridge;

architecture rtl of svec_sfpga_bridge is
  constant c_tag_addr : std_logic_vector(7 downto 0) := x"a0";
  constant c_tag_ds_req : std_logic_vector(7 downto 0) := x"a1";

  signal as_fedge_p : std_logic;
  signal ds_synced: std_logic_vector(1 downto 0);
  
  signal ADDR_latched              : std_logic_vector(31 downto 1);
  signal LWORD_latched_n           : std_logic;
  signal AM_latched                : std_logic_vector(5 downto 0);
  signal WRITE_latched : std_logic;

  signal addr_req : std_logic;
  signal ds_req : std_logic;
  signal wdata_req : std_logic;

  signal ds_count : unsigned(2 downto 0);

  -- Calculate the number of LATCH DS states necessary to match the timing
  -- rule 2.39 page 113 VMEbus specification ANSI/IEEE STD1014-1987.
  -- (max skew for the slave is 20 ns)
  constant c_num_latchDS            : natural range 1 to 8 :=
    (20 + g_CLOCK_PERIOD - 1) / g_CLOCK_PERIOD;

  
begin

  U_Sync1 : gc_sync_ffs
    port map (
      clk_i    => afpga_clk_i,
      rst_n_i  => afpga_rst_n_i,
      data_i   => vme_i.as_n,
      npulse_o => as_fedge_p);

  U_Sync2 : gc_sync_ffs
    port map (
      clk_i    => afpga_clk_i,
      rst_n_i  => afpga_rst_n_i,
      data_i   => vme_i.ds_n(0),
      synced_o => ds_synced(0));

  U_Sync3 : gc_sync_ffs
    port map (
      clk_i    => afpga_clk_i,
      rst_n_i  => afpga_rst_n_i,
      data_i   => vme_i.ds_n(1),
      synced_o => ds_synced(1));

  p_latch_addr: process(afpga_clk_i)
  begin
    if rising_edge(afpga_clk_i) then
      if as_fedge_p = '1' then
        ADDR_latched <= vme_i.Addr;
        LWORD_latched_n <= vme_i.LWORD_n;
        AM_latched <= vme_i.am;
        addr_req <= '1';
      
      end if;
    end if;
  end process;

  p_latch_data: process(afpga_clk_i)
  begin
    if rising_edge(afpga_clk_i) then
      if ds_synced /= "00" then
        WRITE_latched <= vme_i.WRITE_n;
      end if;
    end if;
  end process;

    
     
  
  
  
end rtl;
